#
# spec file for package haveged
#
# Copyright (c) 2023 SUSE LLC
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via https://bugs.opensuse.org/
#


%{!?_udevrulesdir: %global _udevrulesdir %(pkg-config --variable=udevdir udev)/rules.d }
Name:           haveged
Version:        1.9.18
Release:        3.3
Summary:        Daemon for feeding entropy into the random pool
License:        GPL-3.0-only
Group:          System/Daemons
URL:            https://github.com/jirka-h/haveged
Source0:        https://github.com/jirka-h/haveged/archive/v%{version}.tar.gz#/%{name}-%{version}.tar.gz
Source2:        %{name}.service
Source3:        90-haveged.rules
Source4:        haveged-dracut.module
Patch0:         ppc64le.patch
# PATCH-FIX-UPSTREAM: don't write to syslog at startup to avoid deadlocks psimons@suse.com bnc#959237
Patch2:         haveged-no-syslog.patch
Patch3:         harden_haveged.service.patch
BuildRequires:  autoconf
BuildRequires:  automake
BuildRequires:  libtool
BuildRequires:  pkgconfig
BuildRequires:  pkgconfig(systemd)
BuildRequires:  pkgconfig(udev)
Requires(post): coreutils
Requires(postun):coreutils
Enhances:       apache2
Enhances:       gpg2
Enhances:       openssl
Enhances:       openvpn
Enhances:       php5
Enhances:       smtp_daemon
Enhances:       systemd
%{?systemd_requires}

%description
The haveged daemon feeds the Linux entropy pool with random
numbers generated from hidden processor state.

For more information, see http://www.issihosts.com/haveged/ .

%package devel
Summary:        Haveged development files
Group:          Development/Libraries/C and C++
Requires:       libhavege2 = %{version}

%description devel
Headers and for the haveged library

This package contains the haveged implementation of the HAVEGE
algorithm and supporting features.

%package -n libhavege2
Summary:        Haveged interface library
Group:          System/Libraries

%description -n libhavege2
Shared object for the haveged library.
This package contains the haveged implementation of the HAVEGE
algorithm and supporting features.

%prep
%autosetup -p1

%build
autoreconf -fvi
export CFLAGS="%{optflags} -fpie -D_DEFAULT_SOURCE -D_GNU_SOURCE"
export LDFLAGS="-Wl,-z,relro,-z,now -pie"
# ENT randomly fails so disable the test
%configure \
    --disable-static \
    --disable-enttest \
    --enable-nistest \
    --enable-daemon \
    --enable-clock_gettime
make %{?_smp_mflags}

%check
#XXX: nist test is killed by SIGKILL with static int random_pool1[_32MB] on
#     32bit. Let change it to _08MB to avoid the test beeing killed, even if I
#     am not sure allocate of 128M is prohibited
%ifarch %{ix86}
sed -i 's/\[_32MB\]/[_08MB]/' nist/nist.c
%endif

make %{?_smp_mflags} check

%install
%make_install
install -Dpm 0644 %{SOURCE2} \
  %{buildroot}%{_unitdir}/%{name}.service
install -Dpm 0644 %{SOURCE3} \
  %{buildroot}%{_udevrulesdir}/90-%{name}.rules
install -Dpm 0755 %{SOURCE4} \
  %{buildroot}%{_prefix}/lib/dracut/modules.d/98%{name}/module-setup.sh
rm -f %{buildroot}%{_libdir}/libhavege.*a
ln -s %{_sbindir}/service %{buildroot}%{_sbindir}/rc%{name}

%post
%{?udev_rules_update:%udev_rules_update}
%service_add_post %{name}.service
%{?regenerate_initrd_post}

%postun
%service_del_postun %{name}.service
%{?regenerate_initrd_post}

%posttrans
%{?regenerate_initrd_posttrans}

%pre
%service_add_pre %{name}.service

%preun
%service_del_preun %{name}.service

%post -n libhavege2 -p /sbin/ldconfig
%postun -n libhavege2 -p /sbin/ldconfig

%files
%license COPYING
%{_sbindir}/rc%{name}
%{_sbindir}/%{name}
%{_mandir}/man8/%{name}.8%{?ext_man}
%{_unitdir}/%{name}.service
%{_udevrulesdir}/90-%{name}.rules
%dir %{_prefix}/lib/dracut
%dir %{_prefix}/lib/dracut/modules.d
%dir %{_prefix}/lib/dracut/modules.d/98%{name}
%{_prefix}/lib/dracut/modules.d/98%{name}/module-setup.sh

%files devel
%license COPYING
%{_mandir}/man3/libhavege.3%{?ext_man}
%dir %{_includedir}/%{name}
%{_includedir}/%{name}/havege.h
%doc contrib/build/havege_sample.c
%{_libdir}/*.so

%files -n libhavege2
%license COPYING
%{_libdir}/*.so.*

%changelog
* Fri Oct 27 2023 Otto Hollmann <otto.hollmann@suse.com>
- Remove haveged-switch-root.service because it's implemented incorrectly and
  neither upstream don't know how to fix it (#77). On the other hand, without
  this service haveged will be started from scratch after switch root so it's
  hopefully no big deal. Also remove patch for bsc#1203079 as it's considered
  as a security threat because of creating fixed name file in world-writable
  directory. [jsc#PED-6184, bsc#1206699]
  * Remove
  - haveged-switch-root.service
  - haveged-switch-root.patch
* Fri Dec  2 2022 Otto Hollmann <otto.hollmann@suse.com>
- Synchronize haveged instances during switching root (bsc#1203079)
  * Add haveged-switch-root.patch
* Sun Apr 24 2022 Dirk Müller <dmueller@suse.com>
- update to 1.9.18:
  * Round bits up and target full pool size [Jason A. Donenfeld]
  * Specify right kernel commit in README [Jason A. Donenfeld]
* Tue Feb 15 2022 Otto Hollmann <otto.hollmann@suse.com>
- Update to v1.9.17:
  * Added new verbose mode [Jirka Hladky]
  * haveged-once.service - use @SBIN_DIR@ instead of hard-coded path [Jirka Hladky]
- Changes for version v1.9.16:
  * Allow newuname syscall [Jirka Hladky]
  * Fix: haveged cannot be run as an application if also running as a daemon [Günther Brunthaler]
  * Add entropy unconditionally at the start and then every 60 seconds [Jirka Hladky]
  * New parameter --once to refill entropy once and quit immediately [Jirka Hladky]
  * Added haveged-once.service to provide entropy once (intended for initramfs) [Jirka Hladky]
- Changes for version v1.9.15:
  * Check for sys/auxv.h before using it. [Peter Seiderer]
  * fix build on uclibc (origin/pr/58) [Pierre-Jean Texier]
  * Improved make check tests [Jirka Hladky]
  * Removed old init.d files. Configs are under contrib directory [Jirka Hladky]
  * Support for Linux kernel LRNG patch set
* Mon Nov 22 2021 Johannes Segitz <jsegitz@suse.com>
- Remove ProtectKernelTunables hardening, causes the service to fail
  (boo#1192921)
* Tue Nov  2 2021 Marcus Meissner <meissner@suse.com>
- revert last change, e.g. for VMs where we are not being fed entropy
  from the host or similar setups.
* Mon Oct 11 2021 Cristian Rodríguez <crrodriguez@opensuse.org>
- Improvements on the linux kernel random subsystem have made
  the haveged service/daemon obsolete, remove the service files,
  initrd modules and udev rules, the other components
  are still useful.
* Tue Sep 21 2021 Johannes Segitz <jsegitz@suse.com>
- Added hardening to systemd service(s) (bsc#1181400). Added patch(es):
  * harden_haveged.service.patch
  Modified:
  * haveged-switch-root.service
  * haveged.service
* Mon Jan  4 2021 Paolo Stivanin <info@paolostivanin.com>
- Update to 1.9.14:
  * made enttest configurable
  * havegecmd.c - new command added to close the communication socket
- Drop haveged-conditional-enttest.patch
* Tue Jun 30 2020 Paolo Stivanin <info@paolostivanin.com>
- Update to 1.9.13:
  * added support for --version [Jirka Hladky]
  * updated systemd service file [Jirka Hladky]
  * fix a memory leak in havege_destroy [Anakin Zhang]
  * updated version reported by program [Jirka Hladky]
  * fix ordering cycle with private tmp [Christian Hesse]
  * Updated systemd service file [Nicolas Braud-Santoni]
  * Bump soname [Nicolas Braud-Santoni]
  * Fix crash on shutdown in threaded mode [Sergei Trofimovich]
  * Fix compilation with --enable-threads [Lars Wendler]
* Tue May 26 2020 Martin Pluskal <mpluskal@suse.com>
- Update to version 1.9.8:
  * Fix for Unresolved symbol error_exit in libhavege #20 by pld-gitsync [Jirka Hladky]
  * order after systemd-tmpfiles-setup-dev.service (origin/pr/21) [Christian Hesse]
  * use systemd security features [Christian Hesse]
  * do not run in container [Christian Hesse]
  * do not use carriage return in line break [Christian Hesse]
  * Fixed invalid UTF-8 codes in ChangeLog [Jirka Hladky]
- Changes for version 1.9.5:
  * Added test for /dev/random symlink [Jirka Hladky]
  * Update to automake 1.16 [Jirka Hladky]
  * Fix segv at start [Andrew]
  * Fixed built issue on Cygwin [jbaker6953]
  * Fix segfault on arm machines (origin/pr/7) [Natanael Copa]
  * init.d/Makefile.am - add missing dependency [Jackie Huang]
  * service.redhat - update PIDFile [Pierre-Jean Texier]
  * Fix type mismatch in get_poolsize [Andreas Schwab]
  * Fixup upstream changelog [Nicolas Braud-Santoni]
  * Remove support for CPUID on ia64 (origin/pr/19) [Jeremy Bobbio]
  * Output some progress during CUSUM and RANDOM EXCURSION test [Sven Hartge]
  * Diagnostics capture mode now works correctly [Ethan Rahn]
- Drop upstream patches:
  * f2193587.patch
  * get-poolsize.patch
* Tue Jul 30 2019 Andreas Schwab <schwab@suse.de>
- get-poolsize.patch: Fix type mismatch in get_poolsize that breaks error
  checking (bsc#1111047)
* Wed Sep 19 2018 Dr. Werner Fink <werner@suse.de>
- Add patch f2193587.patch from github pull request (bsc#1134351)
  * Fix segfault on arm machines which do not eport the cache size
    or say it is -1 in sysfs
- Refresh patches
* Tue Sep 11 2018 Jan Engelhardt <jengelh@inai.de>
- Use noun phrase for summary. Grammar fixes to description.
* Tue Sep 11 2018 Martin Pluskal <mpluskal@suse.com>
- Update to version 1.9.4:
  * Avoid misleading message if cmd socket is in use
  * Implements a command mode and use it for chroot
- Drop upstreamed patches:
  * change-sysroot.patch
- Update project and download url's
* Mon Jul  2 2018 werner@suse.de
- Add patch change-sysroot.patch to
  move forward to socket communication within private network
- Add haveged-switch-root.service which send the new root location
  to running haveged process
* Wed Jun 27 2018 werner@suse.de
- Make haveged survive switch root within initrd  (boo#1075359)
  added change-sysroot.patch
  haveged-switch-root.service
* Mon Mar 19 2018 kukuk@suse.de
- Use %%license instead of %%doc [bsc#1082318]
* Sun Feb 18 2018 avindra@opensuse.org
- update to 1.9.2
  * Cross compile fixes
  * Limit watermark max to less than pool size to avoid 100%% cpu
    condition
  * Add service.suse init script
- drop fix-enforced-clock-gettime.patch (fixed upstream)
- cleanup with spec-cleaner
* Sun Jun 19 2016 jengelh@inai.de
- reset rpm groups
* Fri Jun 17 2016 psimons@suse.com
- Add haveged-no-syslog.patch to remedy the potential for deadlocks
  when booting the system: journald reads from /dev/random, which
  receives entropy from haveged, which in turn logs to syslog
  before providing any. Ideally, haveged would provide a proper
  command-line flag to disable use of syslog. Will work with
  upstream to resolve this in a cleaner way. (bnc#959237)
- Remove "After=systemd-random-seed.service" from systemd service
  file to avoid the potential for deadlocks when booting the
  system: systemd-random-seed needs /var to read its previous
  state; mounting /var needs journald; journald needs entropy; and
  entropy is provided by haveged, which needs systemd-random-seed.
  (bnc#959237)
* Thu Dec 10 2015 tiwai@suse.de
- Fix missing dependency on coreutils for initrd macros (boo#958562)
* Fri Oct  9 2015 meissner@suse.com
- haveged.service: reverted the dependency on systemd-journald.socket
  for now, as it causes a dependency loop. bsc#949728 bsc#950857
* Wed Sep 30 2015 meissner@suse.com
- modules-setup.sh should be module-setup.sh in dracut, also 755
- add dependency on systemd-journald.socket to make haveged
  really start before journald in the initrd. bsc#898669
* Tue Sep 29 2015 dmueller@suse.com
- add fix-enforced-clock-gettime.patch to fix build on generic
  platforms like e.g. aarch64, which is only needed due to
  our (non-recommended choice) of using gettime (fate#318370)
* Tue Feb 17 2015 meissner@suse.com
- haveged-dracut.module: include haveged into the initrd for
  randomness generation. bsc#898669
- haveged.service: adjust so it is started before journald
  and also make sure it is shutdown quite late. bsc#898669
- regenerate initrd on installation. bsc#898669
* Fri Aug 29 2014 tchvatal@suse.com
- Adjust the service to start early enough for all usecases.
  Should fix bnc#892096.
* Tue Jun 10 2014 tchvatal@suse.com
- Disable haveged service on containers. bnc#881882
* Mon May 12 2014 tchvatal@suse.com
- Disable randomly failing ENT test bnc#876674
- Added patches:
  * haveged-conditional-enttest.patch
* Mon May 12 2014 tchvatal@suse.com
- Threads seem not to fix the random testsuite fail.
* Sat May 10 2014 crrodriguez@opensuse.org
- Fix "stop" of service, the daemon in foreground actually
  exits with a weird exit code. workaround this misbehaviour
  in the service file.
* Sat May 10 2014 crrodriguez@opensuse.org
- revert to non-forking service, "forking" services are racy
  and provided by systemd purely for compatibility with old daemons.
  see daemon(7) for the rationale. Only "simple" (default) and
  "notify" services must be used if possible.
- Also, there is no haveged.conf around, neither such file
  is parsed by the daemon at all.
- Start the daemon as soon as the random device is available with
  the help of udev, as starting services while starved of entropy
  is no good.
* Wed May  7 2014 tchvatal@suse.com
- Cleanup with spec-cleaner
- Adapt the service file to be forking and match what it does for
  me on Gentoo.
- Disable threads as they seem break some times.
- Capitalize summaries to make rpmlint happy.
* Tue Mar 11 2014 dvaleev@suse.com
- Add ppc64le support
- added patches:
  * ppc64le.patch
* Tue Feb 18 2014 crrodriguez@opensuse.org
- version 1.9.1
  * test suite improvements
  * man page updates
  * a few build tweaks
- restore rchaveged link
* Mon Jan  6 2014 mvyskocil@suse.com
- update to 1.8
  * Fix return code for signalled termination.
  * Update init.d/service.fedora - new return code + simple, not forking
  * Add --enable_unitdir as DESTDIR for systemd install
* Mon Jun 17 2013 mvyskocil@suse.com
- update to 1.7c
  * Correct additional run-time test aligment problems on mips.
  * Correct run-time test aligment problems on sparc and mips.
  * Correct ppc detection in build
    deprecates haveged-ppc.patch
- use correct Group for libhavege1
- haveged-devel requires libhavege1
* Sun May  5 2013 crrodriguez@opensuse.org
- Remove PrivateTmp=true here, haveged does not create temporary
  files, either directly or indirectly.
* Thu May  2 2013 crrodriguez@opensuse.org
- haveged 1.7a
  * Correct VPATH issues and modify check target to support
  parallel builds and changes in automake 1.13 test harness.
  Updated sample spec file and other documentation changes.
* Tue Mar 26 2013 mvyskocil@suse.com
- add autoconf to BuildRequires
- explain the need for autoreconf
- tag haveged-ppc.patch
* Wed Mar 20 2013 crrodriguez@opensuse.org
- Remove all sysvinit compatibility.
* Mon Jan 21 2013 ro@suse.de
- spit off devel package (all new files not present before)
- spit off library package to follow shared library policy
- update to 1.7
  * Add havege_status_dump() as basis for haveged diagnositics
  * Provide sample spec to build haveged and haveged-devel
  * Add contrib/build to provide build support
  * Modify build to use libtool
  * Retname base types to alias stdint.h types to allow 'fixes' if necessary
  * Rename all public functions to follow havege_* naming convention
  * Sanitize havege.h to be include file for devel package
  * Add destroy() to havege for library use.
  * Move all remaining static vars to allocated memory.
  * Rework include file structure to eliminate include sequence errors.
  * Fix (rare) segfault in test0 caused by 1.6 alignment fix
- update to  v1.6a
  * Fix typo in configure.ac
* Fri Dec 21 2012 dvaleev@suse.com
- fix powerpc detection (haveged-ppc.patch)
* Sun Dec 16 2012 pascal.bleser@opensuse.org
- update to 1.6:
  * make clock_gettime a build option and correct dependencies when used
  * fix alignment fault on arm64 in procedure A test0
* Mon Nov 26 2012 crrodriguez@opensuse.org
- Current version does support ARM, remove the ExcludeArch
* Mon Sep 24 2012 crrodriguez@opensuse.org
- Tight up systemd service file, this daemon does not
  need network and can use PrivateNetwork=yes
* Mon Aug 13 2012 mvyskocil@suse.cz
- Update to 1.5
  * Add online tests based on AIS-31
  * Add -p option to specify the pid location.
  * Fix -F option to not take argument - i.e. now a switch
  * Factor diagnostic methods for capture and inject for better performance
  * Fix install target, move to bin and eliminate script if not daemon, now use sysv and systemd templates
- use -F with no arguments in haveged.service
- work-around nist check on i586 by decreasing the size of static array in a test
* Tue May 15 2012 aboe@opensuse.org
- Update to version 1.4
  * Add s390 architecture. Thanks to Dan Horak and Jiri Hladky
  * Add generic architecture with clock_gettime() timer.
  * Rewrite collection loop to support multiple instances and new diagnostics
  * Rewrite tuning mechanism to add virtual file system mining and correct cpuid bugs
  * Add prototype multi-thread collection option
  * Reduce collection buffer size to .5MB
  * Improve/correct build and install
  * Add new invocation options to support new features.
- init script additions $remote_fs added
* Thu Feb  9 2012 crrodriguez@opensuse.org
- build with -fpie
* Sat Dec 31 2011 crrodriguez@opensuse.org
- Put the haveged binary back in /usr/sbin not in /sbin
* Wed Dec 14 2011 crrodriguez@opensuse.org
- Update to version 1.3.0a, PPC fixes.
- Cleanup systemd unit and drop capabiltities there.
* Fri Dec  2 2011 coolo@suse.com
- add automake as buildrequire to avoid implicit dependency
* Tue Nov 29 2011 crrodriguez@opensuse.org
- Use proper systemd macros
- reduce verbosity level to the minimum
* Tue Nov 22 2011 crrodriguez@opensuse.org
- Use Service type "simple" in systemd unit
* Tue Nov  1 2011 ro@suse.de
- fix build on ia64, s390, s390x
* Tue Oct 25 2011 pascal.bleser@opensuse.org
- update to 1.3:
  * reorganized to allow its collection mechanism to be better accessed
    directly through the file system: includes the option to suppress the
    daemon interface in the build for circumstances where /dev/random is
    unavailable or its use is inappropriate
* Sat Oct  1 2011 crrodriguez@opensuse.org
- There is no support for arm targets here, may be fixable
  later.
* Sun Jul 24 2011 crrodriguez@opensuse.org
- Update to version 1.2.
* Mon Jun  6 2011 k0da@opensuse.org
- fix ppc64 build
* Mon Jun  6 2011 adrian@suse.de
- fix URL to source tar ball
* Sat May  7 2011 crrodriguez@opensuse.org
- Update to version 1.1, upstream version where the bugs
  present in old versions have been fixed in different ways.
* Tue Mar  1 2011 meissner@suse.de
- avoid unnecessary services. bnc#675841
  also the start should be mediated by YaST or kiwi depending
  on presence of a virtualization environment, not by the package
  itself.
* Tue Jan  4 2011 cristian.rodriguez@opensuse.org
- Update to version 0.9_git201101042303
  * fix wrong usage of strdupa
- fix build in older distros that have no XZ
* Tue Jan  4 2011 cristian.rodriguez@opensuse.org
- Fix build in older distros
- run spec cleaner
* Tue Dec  7 2010 coolo@novell.com
- prereq init script syslog
* Thu Nov 25 2010 cristian.rodriguez@opensuse.org
- Use gcc's __cpuid instead of homegrown asm.
* Tue Nov 23 2010 cristian.rodriguez@opensuse.org
- Fix optimization problem, actually a bug in inline asm
  code. avoid using it, instead try hard to use either
  GCC builtins or properly corrected inline asm.
  Thanks Richard for pointing to the right solution.
* Sat Nov  6 2010 aj@suse.de
- Package /lib/systemd/system/ and /lib/systemd to fix build
* Fri Nov  5 2010 cristian.rodriguez@opensuse.org
- Link with full RELRO (-Wl,-z,relro,-z,now)
* Wed Nov  3 2010 cristian.rodriguez@opensuse.org
- add systemd support
* Fri Sep 24 2010 cristian.rodriguez@opensuse.org
- Drop as much capabilitites as possible using libcap-ng
* Sat Jul 31 2010 cristian.rodriguez@opensuse.org
- I meant Enhances not Supplements
* Fri Jul 30 2010 cristian.rodriguez@opensuse.org
- Implement hack to start by default only in VMs
* Tue Jul 27 2010 cristian.rodriguez@opensuse.org
- Run the complete test suite (for the paranoid)
- use O_CLOEXEC on fds
* Tue Jul 20 2010 cristian.rodriguez@opensuse.org
- enable daemon by default
- add a few Supplements so the it gets installed automatically.
* Sun Jul 18 2010 cristian.rodriguez@opensuse.org
- add proper Requires(pre)
* Fri Jul 16 2010 cristian.rodriguez@opensuse.org
- build with no optimization, there are reports saying it
  may crash with -O1 like http://bugs.debian.org/cgi-bin/bugreport.cgi?bug=563938
* Thu Jul 15 2010 cristian.rodriguez@opensuse.org
- move deamon to /sbin
- tune the spec file
- add a SUSE standard init script
* Fri May 28 2010 meissner@suse.de
- haveged 0.9
  Gather entropy by doing calculation and looking
  at secondary high resolution processor information
  (tsc, cache misses etc.)
